from django.db import models
from django.core.validators import URLValidator
from utils import encode_in_alphabet
import settings


class TooManyUrlsError(Exception):
    """Too many urls per ip address!"""


class Url(models.Model):
    id = models.AutoField(primary_key=True)
    long_url = models.URLField(max_length=2000)
    ip_address = models.IPAddressField(null=True)

    @property
    def short_url(self):
        if self.id is None:
            return '<unsaved>'
        else:
            return encode_in_alphabet(self.id, settings.URL_ALPHABET)

    def __unicode__(self):
        return '%(short_url)s -> %(long_url)s' % {
            'short_url': self.short_url,
            'long_url': self.long_url,
        }

    @classmethod
    def get_or_create_url(cls, long_url, ip_address=None):
        try:
            url = cls.objects.get(long_url=long_url)
        except cls.DoesNotExist:
            # Validate url.
            validate = URLValidator(verify_exists=True)
            validate(long_url)

            # Check limit for urls per ip.
            num_of_urls = Url.objects.filter(ip_address=ip_address).count()
            if num_of_urls >= settings.URLS_PER_IP:
                raise TooManyUrlsError('Too many urls (%d) for ip address %s!'
                    % (num_of_urls, ip_address))

            url = cls(long_url=long_url, ip_address=ip_address)
            url.save()
        return url


class Bookmark(models.Model):
    id = models.AutoField(primary_key=True)
    url = models.URLField(max_length=2000)
    name = models.CharField(max_length=200, unique=True)

    def __unicode__(self):
        return '%(name)s -> %(url)s' % {
            'name': self.name,
            'url': self.url,
        }

    @classmethod
    def get_bookmark(cls, name):
        try:
            bookmark = cls.objects.get(name=name)
        except cls.DoesNotExist:
            bookmark = None
        return bookmark
