/* (c) 2012 Danilo Šegan — JavaScript/Canvas based loader animation.
 *
 * This file is distributed under AGPL version 3 or later.
 */


function Loader(doc, canvas_id) {
    this.document = doc;
    this.total_steps = 8;
    this.current_step = 0;
    this.context = this.getContextForCanvas(canvas_id);
    this._interval = undefined;
}

Loader.prototype = {
    /* Show (and animate) the loader. */
    'show': function() {
        var that = this;
        this.draw();
        this.context.canvas.style.visibility = "visible";
        if (this._interval === undefined) {
            this._interval = setInterval(function() {
                that.current_step = (that.current_step + 1) % that.total_steps;
                that.draw();
            }, 140);
        }
    },

    /* Hide (and stop animating) the loader. */
    'hide': function(success) {
        clearInterval(this._interval);
        this._interval = undefined;
        this.context.canvas.style.visibility = "hidden";
    },

    'getContextForCanvas': function(canvas_id) {
        var canvas = this.document.getElementById(canvas_id);
        if (canvas !== undefined) {
            return canvas.getContext("2d");
        } else {
            throw "Canvas '" + canvas_id + "' not found in the document.";
        }
    },

    /* Clears the entire canvas. */
    'clear': function(width, height) {
        this.context.clearRect(0, 0, width, height);
    },

    /* Figure out the dimensions for drawing the circle of dots. */
    'findCoordinates': function() {
        var width = this.context.canvas.width;
        var height = this.context.canvas.height;
        var diameter = width > height ? height : width;

        // A single dot diameter is derived from the circle perimeter and
        // total number of dots to be drawn as:
        //   dot_d = PI * (d-dot_d) / num_dots / 1.5
        //   === ... === dot_d = PI * d / (1.5 * num_dots + PI)
        var dot_diameter = (
            Math.PI * diameter / (1.5 * this.total_steps + Math.PI));
        return {
            // Maximum loader width.
            'width': width,
            // Maximum loader height.
            'height': height,
            // Final loader diameter (= min(width, height)).
            'diameter': diameter,
            // Center of the circle of dots.
            'center': { 'x': width/2, 'y': height/2 },
            // A diameter of a single animated dot.
            'dot_diameter': dot_diameter,
            // Radius of the circle on which centers of the dots lay.
            // Ensure that full dots are visible in the canvas.
            'dots_centers_radius': (diameter - dot_diameter) / 2,
            // Angle between centers of two adjacent dots.
            'dot_angle': 2 * Math.PI / this.total_steps,
        };

    },

    /* Draws the entire loader. */
    'draw': function() {
        var coordinates = this.findCoordinates();
        this.clear(coordinates.width, coordinates.height);
        for (var step=0; step < this.total_steps; step++) {
            var dot_center = this.findDotCenter(
                coordinates.center, coordinates.dots_centers_radius,
                - step * coordinates.dot_angle);
            var colour = this.getDotColour(this.current_step, step);
            this.drawDot(dot_center, coordinates.dot_diameter, colour);
        }
    },

    /* Draws the dot for step 'index' in appropriate colour. */
    'drawDot': function(center, size, colour) {
        this.context.beginPath();
        this.context.arc(center.x, center.y, size/2, 0, Math.PI*2, true);
        this.context.closePath();
        this.context.fillStyle = colour;
        this.context.fill();
    },

    /* Calculates the dot centre position for given 'center' and 'radius' of
       circle-of-dots and for 'angle' of the particular dot to be drawn. */
    'findDotCenter': function(center, radius, angle) {
        var dot_x = center.x - radius * Math.sin(angle);
        var dot_y = center.y - radius * Math.cos(angle);
        return { 'x': dot_x, 'y': dot_y };
    },

    /* Returns the colour to fill the dot on position 'index'.
     * 'current_step' indicates the currently "active" dot.
     */
    'getDotColour': function(current_step, index) {
        var colours = ['#444444', '#888888', '#aaaaaa', '#eeeeee'];
        if (index === current_step) {
            return colours[0];
        } else if ((current_step - 1) % this.total_steps === index) {
            return colours[1];
        } else if ((current_step - 2) % this.total_steps === index) {
            return colours[2];
        } else {
            return colours[3];
        }
    }


};
