from django.test import TestCase
from django.core.exceptions import ValidationError
from django.db import IntegrityError
from url_shortener.models import (
    Bookmark,
    TooManyUrlsError,
    Url,
)
import utils
import settings


class TestUtils(TestCase):
    def test_encode_in_alphabet(self):
        self.assertEqual('f', utils.encode_in_alphabet(5, 'abcdef'))

        big_number = 9878473298476238723492482394723482
        self.assertEqual(
            hex(big_number),
            '0x%sL' % utils.encode_in_alphabet(big_number, '0123456789abcdef')
        )

    def test_decode_in_alphabet(self):
        self.assertEqual(5, utils.decode_in_alphabet('bab', 'ab'))
        self.assertEqual(-1, utils.decode_in_alphabet('S', 'ab'))


class TestUrl(TestCase):
    def test_get_or_create_url_success(self):
        url = Url.get_or_create_url(long_url='http://gnome.org')
        self.assertEqual('http://gnome.org', url.long_url)
        self.assertNotEqual('<unsaved>', url.short_url)

    def test_get_or_create_url_reuse(self):
        url1 = Url.get_or_create_url(long_url='http://ubuntu.com')
        url2 = Url.get_or_create_url(long_url='http://ubuntu.com')
        self.assertEqual(url1, url2)

    def test_get_or_create_url_broken_link(self):
        self.assertRaises(
            ValidationError,
            Url.get_or_create_url,
            {'long_url': 'http://x.not_exist'}
        )

    def test_get_or_create_url_invalid_url(self):
        self.assertRaises(
            ValidationError,
            Url.get_or_create_url,
            {'long_url': 'not_valid'}
        )

    def test_get_or_create_url_too_many_urls_per_ip(self):
        old_limit = settings.URLS_PER_IP
        settings.URLS_PER_IP = 1
        my_ip = '127.0.0.2'
        Url.get_or_create_url(long_url='http://ubuntu.com', ip_address=my_ip)
        with self.assertRaises(TooManyUrlsError):
            Url.get_or_create_url(long_url='http://gnome.org', ip_address=my_ip)
        settings.URLS_PER_IP = old_limit


class TestBookmark(TestCase):
    def test_get_bookmark_not_exists(self):
        self.assertIsNone(Bookmark.get_bookmark(name='bookmark_test'))

    def test_get_bookmark_not_exists(self):
        bookmark = Bookmark(name='bookmark')
        bookmark.save()
        self.assertEqual(bookmark, Bookmark.get_bookmark(name='bookmark'))

    def test_bookmark_name_uniqu(self):
        bookmark = Bookmark(name='bookmark')
        bookmark.save()
        with self.assertRaises(IntegrityError):
            Bookmark(name='bookmark').save()


class TestHomePageView(TestCase):
    def test_home_page_view(self):
        response = self.client.get('/')
        self.assertEqual(200, response.status_code)


class TestUrlCreationView(TestCase):
    def test_url_creation_view_success(self):
        response = self.client.post(
            '/create/', {'long_url': 'http://gnome.org'}, HTTP_HOST='test')
        self.assertEqual(200, response.status_code)
        self.assertEqual(
            '{"short_url": "http://test/%s"}' % settings.URL_ALPHABET[1],
            response.content
        )

    def test_url_creation_view_no_param(self):
        response = self.client.post('/create/')
        self.assertEqual(400, response.status_code)
        self.assertEqual(response.content, 'Unknown url!')

    def test_url_creation_view_url_no_exist(self):
        response = self.client.post('/create/', {'long_url': 'not_exist'})
        self.assertEqual(400, response.status_code)
        self.assertEqual(response.content, 'Not a valid url!')

    def test_url_creation_view_invalid_url(self):
        old_limit = settings.URLS_PER_IP
        settings.URLS_PER_IP = 0
        response = self.client.post(
            '/create/', {'long_url': 'http://gnome.org'})
        self.assertEqual(400, response.status_code)
        self.assertEqual(
            'Too many urls (0) for ip address 127.0.0.1!', response.content)
        settings.URLS_PER_IP = old_limit


class TestUrlRedirectionView(TestCase):
    def test_url_redirection_not_exists(self):
        response = self.client.get('/not_exists/')
        self.assertEqual(404, response.status_code)

    def test_url_redirection_exists(self):
        url = Url.get_or_create_url(long_url='http://ubuntu.com')
        response = self.client.get('/' + url.short_url + '/')
        self.assertEqual(301, response.status_code)
        self.assertEqual('http://ubuntu.com', response['location'])
